// Window class implementation
#include "pch.h"

#include "Window.h"
#include "util.h"

namespace nf::cli {
    Window::Window(unsigned int width, unsigned int height)
        : m_handle(nullptr)
        , m_wndClassName("NothinFancyWindow")
        , m_windowedStyle(WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX)
        , m_width(width)
        , m_height(height)
    {
        NFLog("Creating window");

        // Register window class
        WNDCLASS wc = {};
        wc.lpszClassName = m_wndClassName;
        wc.hCursor = LoadCursor(nullptr, IDC_ARROW);
        wc.lpfnWndProc = wndProc;
        RegisterClass(&wc);

        m_handle = CreateWindow(m_wndClassName, "NF Window", m_windowedStyle, 0, 0, 0, 0, nullptr, nullptr, nullptr, this);
        setSize();
    }

    void Window::runLoop() {
        show();
        MSG msg = {};
        while (GetMessage(&msg, nullptr, NULL, NULL)) {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }

    void Window::show(bool show) {
        ShowWindow(m_handle, show ? SW_SHOW : SW_HIDE);
    }

    LRESULT CALLBACK Window::wndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
        static Window* wnd = nullptr;

        switch (msg) {
            case WM_CREATE: {
                wnd = reinterpret_cast<Window*>(reinterpret_cast<CREATESTRUCT*>(lParam)->lpCreateParams);

                // Temporary monitor choice
                POINT cursor = {};
                GetCursorPos(&cursor);
                MONITORINFO mi = {};
                mi.cbSize = sizeof(mi);
                GetMonitorInfo(MonitorFromPoint(cursor, MONITOR_DEFAULTTONEAREST), &mi);
                int monWidth = mi.rcMonitor.right - mi.rcMonitor.left, monHeight = mi.rcMonitor.bottom - mi.rcMonitor.top;
                int x = mi.rcMonitor.left + (monWidth / 2) - (wnd->m_width / 2), y = mi.rcMonitor.top + (monHeight / 2) - (wnd->m_height / 2);
                SetWindowPos(hWnd, nullptr, x , y , 0, 0, SWP_NOZORDER | SWP_NOSIZE);
                return 0;
            }

            case WM_DPICHANGED: {
                // Prevents resize on DPI change
                wnd->setSize();
                return 0;
            }

            case WM_CLOSE:
                DestroyWindow(hWnd);
                return 0;

            case WM_DESTROY:
                PostQuitMessage(0);
                return 0;
        }

        return DefWindowProc(hWnd, msg, wParam, lParam);
    }

    void Window::setSize() {
        RECT cli = {};
        cli.right = m_width;
        cli.bottom = m_height;
        AdjustWindowRectExForDpi(&cli, m_windowedStyle, FALSE, NULL, GetDpiForWindow(m_handle));
        unsigned int width = cli.right - cli.left, height = cli.bottom - cli.top;
        SetWindowPos(m_handle, nullptr, 0, 0, width, height, SWP_NOZORDER | SWP_NOMOVE);
    }
}
